@@ -21,8 +21,10 @@
 #include "snd_efw.h"
 #include "snd_dg00x.h"
 #include "snd_motu.h"
+#include "snd_tscm.h"
 
-void hinawa_fw_unit_ioctl(HinawaFwUnit *self, int req, void *args, int *err);
+void hinawa_fw_unit_ioctl(HinawaFwUnit *self, unsigned long req, void *args,
+			  int *err);
 void hinawa_fw_resp_handle_request(HinawaFwResp *self,
 				   struct fw_cdev_event_request2 *event);
 void hinawa_fw_req_handle_response(HinawaFwReq *self,
@@ -32,6 +34,9 @@ void hinawa_snd_unit_write(HinawaSndUnit *self,
 			   const void *buf, unsigned int length,
 			   GError **exception);
 
+void hinawa_snd_unit_ioctl(HinawaSndUnit *self, unsigned long request,
+			   void *arg, GError **exception);
+
 #if HAVE_SND_DICE
 void hinawa_snd_dice_handle_notification(HinawaSndDice *self,
 					 const void *buf, unsigned int len);
@@ -48,5 +53,9 @@ void hinawa_snd_dg00x_handle_msg(HinawaSndDg00x *self, const void *buf,
 void hinawa_snd_motu_handle_notification(HinawaSndMotu *self,
 					 const void *buf, unsigned int len);
 #endif
+#if HAVE_SND_TSCM
+void hinawa_snd_tscm_handle_control(HinawaSndTscm *self, const void *buf,
+				    unsigned int len);
+#endif
 
 #endif
