@@ -275,6 +275,18 @@ void hinawa_snd_unit_write(HinawaSndUnit *self,
 		raise(exception, errno);
 }
 
+void hinawa_snd_unit_ioctl(HinawaSndUnit *self, unsigned long request,
+			   void *arg, GError **exception)
+{
+	HinawaSndUnitPrivate *priv;
+
+	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
+	priv = hinawa_snd_unit_get_instance_private(self);
+
+	if (ioctl(priv->fd, request, arg) < 0)
+		raise(exception, errno);
+}
+
 static void handle_lock_event(HinawaSndUnit *self,
 			      void *buf, unsigned int length)
 {
@@ -289,8 +301,8 @@ static void handle_lock_event(HinawaSndUnit *self,
 
 static gboolean prepare_src(GSource *src, gint *timeout)
 {
-	/* Use blocking poll(2) to save CPU usage. */
-	*timeout = -1;
+	// Use 500 msec for safe cancellation of thread.
+	*timeout = 500;
 
 	/* This source is not ready, let's poll(2) */
 	return FALSE;
@@ -358,6 +370,12 @@ static gboolean check_src(GSource *gsrc)
 		hinawa_snd_motu_handle_notification(HINAWA_SND_MOTU(unit),
 						    priv->buf, len);
 #endif
+#if HAVE_SND_TSCM
+	else if (HINAWA_IS_SND_TSCM(unit) &&
+		 common->type == SNDRV_FIREWIRE_EVENT_TASCAM_CONTROL)
+		hinawa_snd_tscm_handle_control(HINAWA_SND_TSCM(unit),
+						    priv->buf, len);
+#endif
 end:
 	/* Don't go to dispatch, then continue to process this source. */
 	return FALSE;
@@ -419,7 +437,8 @@ void hinawa_snd_unit_listen(HinawaSndUnit *self, GError **exception)
 	priv->len = getpagesize();
 
 	((SndUnitSource *)src)->tag =
-		hinawa_context_add_src(src, priv->fd, G_IO_IN, exception);
+		hinawa_context_add_src(HINAWA_CONTEXT_TYPE_SND, src, priv->fd,
+				       G_IO_IN, exception);
 	if (*exception != NULL) {
 		g_free(buf);
 		g_source_destroy(src);
@@ -463,7 +482,7 @@ void hinawa_snd_unit_unlisten(HinawaSndUnit *self)
 	if (priv->streaming)
 		ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_UNLOCK, NULL);
 
-	g_source_destroy((GSource *)priv->src);
+	hinawa_context_remove_src(HINAWA_CONTEXT_TYPE_SND, (GSource *)priv->src);
 	g_free(priv->src);
 	priv->src = NULL;
 
